@@ -16,6 +16,7 @@ import zipfile
 import tempfile
 import glob
 import re
+from datetime import date
 
 # Third party imports
 import pandas as pd
@@ -169,8 +170,7 @@ def jprint(*args, **kwargs):
 def toogle_code(**kwargs):
     """
     FOR JUPYTER NOTEBOOK ONLY
-    Hide code with a clickable link in a j
-    upyter notebook
+    Hide code with a clickable link in a jupyter notebook
     """
     # Function specific third party import
     try:
@@ -247,6 +247,24 @@ def _hide_traceback(
     )
 
 
+def init_notebook(
+    author="author",
+    creation_date="XXXX/XX/XX",
+    color="blue",
+):
+    """
+    Display a basic initialization message for Jupyter notebooks
+    """
+    cprint(author, color=color)
+    cprint(f"Starting date : {creation_date}", color=color)
+    cprint(
+        "Last modification date : {}/{:02}/{:02}".format(
+            date.today().year, date.today().month, date.today().day
+        ),
+        color=color,
+    )
+
+
 # ~~~~~~~ PREDICATES ~~~~~~~#
 
 
@@ -614,9 +632,9 @@ def head(
     n=10,
     ignore_comment_line=False,
     comment_char="#",
-    max_char_line=200,
+    max_char_line=300,
     sep="\t",
-    max_char_col=50,
+    max_char_col=200,
     **kwargs,
 ):
     """
@@ -1427,7 +1445,11 @@ def qsub(
         return random.randint(0, 100000)
     else:
         stdout = bash(cmd=f"qsub {script_fn}", ret_stdout=True, print_stdout=False)
-        return stdout.split(" ")[2]
+        try:
+            return stdout.split(" ")[2]
+        except Exception as E:
+            cprint("ERROR: job not submitted", color=red)
+            cprint(str(E))
 
 
 def bsub(
@@ -2802,3 +2824,86 @@ def bam_align_summary(fp, min_mapq=30):
                         counter_dict[label]["primary high mapq"] += 1
 
     return pd.DataFrame(counter_dict)
+
+
+def _convert_sec_to_h(s):
+    """
+    Convert seconds to hour, minutes, secondes
+    """
+    s = s % (24 * 3600)
+    h = s // 3600
+    s %= 3600
+    m = s // 60
+    s %= 60
+    return (int(h), int(m), int(s))
+
+
+def line_profiler_parse(fn, percent_cufoff=0.1, sort_by_time=True):
+    """
+    Parse the output of python line_profiler and display for jupyter
+    """
+
+    # Define data header and type
+    header = ["Line #", "Hits", "Time", "Time Per Hit", "% Time", "Line Contents"]
+    dtypes = [int, int, float, float, float, str]
+    dtypes_header = {i: j for i, j in zip(header, dtypes)}
+
+    all_results = []
+
+    with open(fn) as fp:
+        try:
+            while True:
+                line = next(fp).strip()
+                if line.startswith("Total time: "):
+                    time = float(line.split(":")[-1].strip().split(" ")[0])
+                    file = next(fp).split("/")[-1].strip()
+                    function = next(fp).split(":")[-1].strip()
+
+                    # Flush out useles lines
+                    for i in range(3):
+                        _ = next(fp)
+
+                    # Parse data lines
+                    data_lines = []
+                    while True:
+                        line = next(fp).strip()
+                        if line == "":
+                            break
+                        # parse messy lprun lines
+                        try:
+                            ls = line.split()
+                            parsed_line = []
+                            if len(ls) >= 6:
+                                for e, t in zip(
+                                    ls[0:5], (int, int, float, float, float)
+                                ):
+                                    parsed_line.append(t(e))
+                            parsed_line.append(" ".join(ls[5:]))
+                            data_lines.append(parsed_line)
+                        except:
+                            pass
+
+                    if data_lines:
+                        df = pd.DataFrame(data=data_lines, columns=header)
+                        if sort_by_time:
+                            df.sort_values("% Time", inplace=True, ascending=False)
+                        df = df[df["% Time"] >= percent_cufoff]
+                    all_results.append(
+                        {"time": time, "file": file, "function": function, "df": df}
+                    )
+
+        except StopIteration:
+            if data_lines:
+                df = pd.DataFrame(data=data_lines, columns=header)
+                df.sort_values("% Time", inplace=True, ascending=False)
+                df = df[df["% Time"] > 0]
+            all_results.append(
+                {"time": time, "file": file, "function": function, "df": df}
+            )
+
+        all_results.sort(key=lambda item: item.get("time"), reverse=True)
+        for i in all_results:
+            cprint(f"Function {i['function']} from {i['file']}", color="blue")
+            h, m, s = _convert_sec_to_h(i["time"])
+            cprint(f"Execution time: {h}h:{m}m:{s}s", color="red")
+            display(i["df"])
